import os
import json
from vtt2json.vtt2json import to_json

SUBTITLES_DIR = os.path.join(os.path.dirname(__file__), "subtitles")


def get_file(filename):
    return os.path.join(SUBTITLES_DIR, filename)


def test_ignore_lines():
    expected_value = {
        "parsed_lines": [],
    }
    parsed_value = to_json(get_file("ignore_lines.vtt"))

    assert parsed_value == json.dumps(expected_value)


def test_ignore_lines_verbos():
    expected_value = {
        "parsed_lines": [],
        "ignored_lines": [
            "WEBVTT\n",
            "\n",
            "STYLE\n",
            "::cue() {\n",
            "  font-family: Arial, Helvetica, sans-serif;\n",
            "}\n",
        ],
    }
    parsed_value = to_json(get_file("ignore_lines.vtt"), verbos=True)

    assert parsed_value == json.dumps(expected_value)


def test_regular_lines():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:00:01.960",
                "end": "00:00:03.795",
                "text": "CLAIRE: Kids! Breakfast!",
            }
        ],
    }
    parsed_value = to_json(get_file("regular_lines.vtt"))

    assert parsed_value == json.dumps(expected_value)


def test_regular_lines_verbos():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:00:01.960",
                "end": "00:00:03.795",
                "text": "CLAIRE: Kids! Breakfast!",
                "_raw": {
                    "start": "00:00:01.960",
                    "end": "00:00:03.795",
                    "text": "CLAIRE: Kids! Breakfast!",
                },
            }
        ],
        "ignored_lines": [],
    }
    parsed_value = to_json(get_file("regular_lines.vtt"), verbos=True)

    assert parsed_value == json.dumps(expected_value)


def test_multiple_lines():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:00:22.231",
                "end": "00:00:25.526",
                "text": "All right, that's not gonna happen.",
            },
            {
                "start": "00:00:22.231",
                "end": "00:00:25.526",
                "text": "And, wow, you're not wearing that outfit.",
            },
        ],
    }
    parsed_value = to_json(get_file("multiple_lines.vtt"))

    assert parsed_value == json.dumps(expected_value)


def test_multiple_lines_verbos():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:00:22.231",
                "end": "00:00:25.526",
                "text": "All right, that's not gonna happen.",
                "_raw": {
                    "start": "00:00:22.231",
                    "end": "00:00:25.526",
                    "text": "All right, that's not gonna happen.",
                },
            },
            {
                "start": "00:00:22.231",
                "end": "00:00:25.526",
                "text": "And, wow, you're not wearing that outfit.",
                "_raw": {
                    "start": "00:00:22.231",
                    "end": "00:00:25.526",
                    "text": "And, wow, you're not wearing that outfit.",
                },
            },
        ],
        "ignored_lines": [],
    }
    parsed_value = to_json(get_file("multiple_lines.vtt"), verbos=True)

    assert parsed_value == json.dumps(expected_value)


def test_separated_lines():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:02:41.161",
                "end": "00:02:43.956",
                "text": "We just adopted her from Vietnam",
            },
            {
                "start": "00:02:44.039",
                "end": "00:02:46.250",
                "text": "and we're bringing her home",
            },
            {
                "start": "00:02:44.039",
                "end": "00:02:46.250",
                "text": "for the first time.",
            },
        ],
    }
    parsed_value = to_json(get_file("separated_lines.vtt"))

    assert parsed_value == json.dumps(expected_value)


def test_separated_lines_verbos():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:02:41.161",
                "end": "00:02:43.956",
                "text": "We just adopted her from Vietnam",
                "_raw": {
                    "start": "00:02:41.161",
                    "end": "00:02:43.956",
                    "text": "We just adopted her from Vietnam",
                },
            },
            {
                "start": "00:02:44.039",
                "end": "00:02:46.250",
                "text": "and we're bringing her home",
                "_raw": {
                    "start": "00:02:44.039",
                    "end": "00:02:46.250",
                    "text": "and we're bringing her home",
                },
            },
            {
                "start": "00:02:44.039",
                "end": "00:02:46.250",
                "text": "for the first time.",
                "_raw": {
                    "start": "00:02:44.039",
                    "end": "00:02:46.250",
                    "text": "for the first time.",
                },
            },
        ],
        "ignored_lines": ["\n"],
    }
    parsed_value = to_json(get_file("separated_lines.vtt"), verbos=True)

    assert parsed_value == json.dumps(expected_value)


def test_join_separated_lines():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:02:41.161",
                "end": "00:02:46.250",
                "text": "We just adopted her from Vietnam and we're bringing her home for the first time.",
            },
        ],
    }
    parsed_value = to_json(get_file("separated_lines.vtt"), join_separated_lines=True)

    assert parsed_value == json.dumps(expected_value)


def test_join_separated_lines_verbos():
    expected_value = {
        "parsed_lines": [
            {
                "start": "00:02:41.161",
                "end": "00:02:46.250",
                "text": "We just adopted her from Vietnam and we're bringing her home for the first time.",
                "_raw": [
                    {
                        "start": "00:02:41.161",
                        "end": "00:02:43.956",
                        "text": "We just adopted her from Vietnam",
                    },
                    {
                        "start": "00:02:44.039",
                        "end": "00:02:46.250",
                        "text": "and we're bringing her home",
                    },
                    {
                        "start": "00:02:44.039",
                        "end": "00:02:46.250",
                        "text": "for the first time.",
                    },
                ],
            },
        ],
        "ignored_lines": ["\n"],
    }
    parsed_value = to_json(
        get_file("separated_lines.vtt"), verbos=True, join_separated_lines=True
    )

    assert parsed_value == json.dumps(expected_value)



import json
import re
import copy


def _is_ignored_line(vtt_line: str) -> bool:
    ignore_patterns = [r"^WEBVTT", r"^STYLE$", r"::cue", r"^ ", r"^\}$", r"^\n$"]

    for ip in ignore_patterns:
        if re.match(ip, vtt_line):
            return True

    return False


def _is_timestamp_line(vtt_line: str) -> bool:
    timestamp_pattern = r"\d{2}:\d{2}:\d{2}.\d{3}"
    timestamp_line_pattern = rf"^{timestamp_pattern} --> {timestamp_pattern}"

    if re.match(timestamp_line_pattern, vtt_line):
        return True
    else:
        return False


def _extract_timestamps(vtt_line: str) -> list:
    timestamp_pattern = r"\d{2}:\d{2}:\d{2}.\d{3}"
    timestamps: tuple = re.match(
        rf"^({timestamp_pattern}) --> ({timestamp_pattern})", vtt_line
    ).groups()
    return timestamps


def _erase_lf_code(vtt_line: str) -> str:
    return re.sub(r"^(.*)\n$", "\\1", vtt_line)


def _is_end_of_line(text: str) -> list:
    return re.match(r".*\W$", text)


def _join_separated_lines(separated_lines: list) -> list:
    joined_lines = []
    text_queue = []
    start = None
    end = None
    text = None
    raw = []

    for parsed_line in separated_lines:

        text_queue.append(parsed_line["text"])

        if start is None:
            start = parsed_line["start"]

        if "_raw" in parsed_line:
            raw.append(parsed_line["_raw"])

        if _is_end_of_line(parsed_line["text"]):
            end = parsed_line["end"]
            text = " ".join(text_queue)
            text_queue.clear()
            joined_line = {"start": start, "end": end, "text": text}

            if len(raw) > 0:
                joined_line["_raw"] = raw

            joined_lines.append(joined_line)

            # 初期化
            start = None
            end = None
            text = None
            raw = []

    return joined_lines


def to_json(vtt_path: str, verbos=False, join_separated_lines=False) -> str:
    return json.dumps(to_dict(vtt_path, verbos, join_separated_lines))


def to_dict(vtt_path: str, verbos=False, join_separated_lines=False) -> dict:
    parsed_lines = []
    ignored_lines = []
    parsed_line = {"start": None, "end": None, "text": None}

    if verbos:
        parsed_line["_raw"] = {"start": None, "end": None, "text": None}

    with open(vtt_path, "r") as vtt_file:

        for vtt_line in vtt_file:

            if _is_ignored_line(vtt_line):
                ignored_lines.append(vtt_line)

            elif _is_timestamp_line(vtt_line):
                timestamps: tuple = _extract_timestamps(vtt_line)
                start = timestamps[0]
                end = timestamps[1]
                parsed_line["start"] = start
                parsed_line["end"] = end

                if verbos:
                    parsed_line["_raw"]["start"] = start
                    parsed_line["_raw"]["end"] = end

            else:
                text = _erase_lf_code(vtt_line)
                parsed_line["text"] = text

                if verbos:
                    parsed_line["_raw"]["text"] = text

                # 値渡しする (参照渡しするとparsed_linesの中身が上書きされてしまう)
                # ※deepcopyでないと子dict(_raw)が値渡しにならない)
                parsed_lines.append(copy.deepcopy(parsed_line))

    if join_separated_lines:
        parsed_lines = _join_separated_lines(parsed_lines)

    dict_data = {
        "parsed_lines": parsed_lines,
    }

    if verbos:
        dict_data["ignored_lines"] = ignored_lines

    return dict_data
